//-*-C++-*-  
#include <gtk--/style.h>
#include <gdk--/font.h>
#include <gdk--/pixmap.h>
#include <gdk--/window.h>

#include <gtk--/drawingarea.h>
#include <iostream>
#include <math.h>
#include <vector>
#include <string>

#include <CS.h>
#include <PMain.h>
#include <Global.h>

extern "C" {
#include <gdkrotated.h>
#include <stdio.h>
}

#ifdef WITH_DMALLOC 
#include<dmalloc.h>
#endif

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

CS::CS() : pix(0) 
{
 /* ---- Set default values ---- */
 minX = minY = (double) 0.0;
 maxX = (double) 1.0;
 maxY = (double) 1.0;
 xticks = 4;
 yticks = 4;
 xbigTicks = 2;
 ybigTicks = 2;
 xLabelLen = yLabelLen = 3;
 allocGC = true;

}

CS::~CS()
{
#ifndef NDEBUG
 cerr << "Destroy CS" << endl;
#endif

 /* ---- Free Window handle ---- */
 if (win.connected()) win.release();

 /* ---- Check for opened font and free it if needed ---- */
 if (font.connected()) font.release();

 /* ---- Free GC's if needed ---- */
 if (whiteGC.connected()) whiteGC.release();
 if (blackGC.connected()) blackGC.release();
 if (pixGC.connected()) pixGC.release();

#ifndef NDEBUG
 cerr << "End of destroy CS" << endl;
#endif

}
void CS::calcAxisInfo(double& min, double& max, long& ticks, long& bigTicks)
{
 typedef struct _tickInfo_ {
                            double diff;
                            long ticks;
                            long bigTicks;
                           } TickInfo; 

 static float minVal = -1.00;
 static float maxVal =  1.00;
 static float lessZero = -0.1;

 /* ---- Table of values for rounding 'min' and 'max' ---- */
 static float rangeVals[] = {minVal, 
                             -0.80, -0.75, -0.60, -0.40, -0.50, -0.30, 
                             -0.25, -0.15, lessZero,
                             +0.00,
                             +0.10, +0.15, +0.25, +0.30, +0.40, +0.50, +0.60, 
                             +0.75, +0.80,
                             maxVal};

 /* ---- Tick info for all distances between all 'rangeVals' ---- */
 static double maxDiff = (double) 5.00;
 static TickInfo rangeDiff[] = {{0.00, 0, 0}, {0.05, 3, 2}, {0.10, 5, 2}, 
                                {0.15, 3, 2}, {0.20, 5, 2}, {0.25, 5, 5},
                                {0.30, 6, 2}, {0.35, 3, 2}, {0.40, 4, 2},
                                {0.45, 4, 2}, {0.50, 5, 2}, {0.55, 3, 2},
                                {0.60, 6, 2}, {0.65, 3, 2}, {0.70, 3, 2},
                                {0.75, 5, 2}, {0.80, 4, 2}, {0.85, 3, 2},
                                {0.90, 3, 2}, {1.00, 5, 2}, {1.05, 3, 2}, 
                                {1.10, 3, 2}, {1.15, 3, 2}, {1.20, 3, 2}, 
                                {1.25, 6, 3}, {1.30, 3, 2}, {1.35, 3, 2}, 
                                {1.40, 3, 2}, {1.50, 4, 2}, {1.55, 3, 2}, 
                                {1.60, 3, 2}, {1.75, 8, 4}, {1.80, 3, 2}, 
                                {2.00, 5, 2}, {maxDiff, 2, 1}
                               };

 /* ---- Decimal places of 'min' and 'max' ---- */
 int minPlaces = 0; 
 int maxPlaces = 0; 
 int places = 0;

 int i;
 double a = -1.0;
 double m = -1.0;

 /* ---- Check if we have to swap 'min' and 'max' ---- */
 if (min > max) {

  /* ---- Swap it ---- */
  double tmp = min;
  min = max ; max = tmp;

 }

#ifndef NDEBUG 
 cerr << endl << endl;
 cerr << "Min : " << min << " Max: " << max << endl;
#endif

 /* ---- Check if 'min' is near zero ---- */
 if (fabs(min) > CS::epsilon) { 

  /* ---- Calculate places of 'min' ---- */
  minPlaces = (int) floor(log10(fabs(min))) + 1;

#ifndef NDEBUG
  cerr << "MinPlaces: " << minPlaces << endl;
#endif

 } else {

  /* ---- Zero places ---- */
  minPlaces = 0;

 }

 /* ---- Check if 'max' is near zero ---- */
 if (fabs(max) > CS::epsilon) { 

  /* ---- Calculate places of 'max' ---- */
  maxPlaces = (int) floor(log10(fabs(max))) + 1;

#ifndef NDEBUG
  cerr << "MaxPlaces: " << maxPlaces << endl;
#endif

 } else {

  /* ---- Zero places ---- */
  maxPlaces = 0;

 }

 /* ---- Use the maximum as number of places ---- */
 places = (minPlaces < maxPlaces) ? maxPlaces : minPlaces;
 
#ifndef NDEBUG
 cerr << "Places used: " << places << endl;
#endif

 /* ---- Check if 'min' is near zero ---- */
 if (fabs(min) > CS::epsilon) { 

#ifndef NDEBUG
  cerr << "Min big enough" << endl;
#endif

  /* ---- Scale min and max to interval [0.0 .. 1.0] ---- */
  m = min / pow(10, (double) places);

  /* ---- Look for index that will scale 'min' properly ---- */
  i = 0; 
  do {

   i++;

  } while ((rangeVals[i] <= m) && (rangeVals[i] != maxVal));

  /* ---- Check for "floored" value ---- */
  if (rangeVals[i] != minVal) {

   /* ---- Scale smallest val ngt. then 'min' to 'minPlaces' places ---- */
   m = rangeVals[i - 1];
   min = m * pow(10, (double) places);

  } else {
 
   /* ---- Use scaled 'minVal' as 'min' ---- */
   m =   minVal;
   min = minVal * pow(10, (double) places);

  }

#ifndef NDEBUG
  cerr << "Scaled Min: " << m << endl;
#endif

 } else {

#ifndef NDEBUG
  cerr << "Min small" << endl;
#endif

  /* ---- Check if min is positive ---- */
  if (min >= 0.0) {

   /* ---- A valid supremum is 0.0 ---- */
   min = (double) 0.0;
   m = (double) 0.0;

  } else {

   /* ---- A valid supremum is one step below 0.0 ---- */
   min = lessZero * pow(10, (double) places);
   m = lessZero;

  }

 }

 /* ---- Check if 'max' is near zero ---- */
 if (fabs(max) > CS::epsilon) { 

#ifndef NDEBUG
  cerr << "Max big enough" << endl;
#endif

  /* ---- Scale 'max' to interval [0.0 .. 1.0] ---- */
  a = max / pow(10, (double) places);

  /* ---- Look for index that will scale 'max' properly ---- */
  i = 0; 
  do {

   i++;

  } while ((rangeVals[i] < a) && (rangeVals[i] != maxVal));

  /* ---- Check for "floored" value ---- */
  if (rangeVals[i] != maxVal) {

   /* ---- Scale smallest val gt. then 'max' to 'minPlaces' places ---- */
   a = rangeVals[i];
   max = a * pow(10, (double) places);

  } else {
 
   /* ---- Use scaled 'maxVal' as min ---- */  
   max = maxVal * pow(10, (double) places);

  }

#ifndef NDEBUG
  cerr << "Scaled Max: " << a << endl;
#endif

 } else {

#ifndef NDEBUG
  cerr << "Max small" << endl;
#endif

  /* ---- Check if 'max' is positive ---- */
  if (max >= 0.0) {

   /* ---- A valid supremum is 0.0 ---- */
   max = (double) 0.0;
   a = (double) 0.0;

  } else {

   /* ---- A valid supremum is one step below 0.0 ---- */
   max = lessZero * pow(10, (double) maxPlaces);
   a = lessZero;

  }

 }

#ifndef NDEBUG
 cerr <<  "Rounded Min: " << min << " Rounded Max: " << max << endl;
#endif

 /* ---- Calc difference between scaled 'min' and 'max' and init i ---- */
 double diff = a - m;
 i = 0;

#ifndef NDEBUG
 cerr << "Use difference " << diff << endl;
#endif

 /* ---- Look for the difference ---- */
 while ((fabs(diff - rangeDiff[i].diff) > CS::epsilon) && 
        (rangeDiff[i].diff != maxDiff)) {

  /* ---- Print rangeValue and inc index ---- */
#ifndef NDEBUG
  cerr << "RangeVal "  << rangeDiff[i].diff << 
          "  Difference " << fabs(diff - rangeDiff[i].diff) << endl;
#endif

  i++;

 }

 /* ---- Check whether we found a valid difference (This should happen) ---- */
 if (rangeDiff[i].diff == maxDiff) { 

  /* ---- Give a warning ---- */
  cerr << "No valid difference for (" 
       << diff 
       << ") found! Using defaults" 
       << endl;

  /* --- Set defaults ---- */
  ticks = 2;
  bigTicks = 3;

 } else {

  /* ---- Set tick info ----- */
  ticks = rangeDiff[i].ticks;
  bigTicks = rangeDiff[i].bigTicks;

 }

#ifndef NDEBUG
 cerr << "Set Ticks to : " << ticks 
      << " BigTicks to : " << bigTicks 
      << endl << endl;
#endif

}

int CS::setData(Curve &data)
{
 /* ---- Copy plot data ---- */
 toPlot = data;

 /* ---- Check if we have data ---- */
 if (data.size() > 0 ) {

  /* ---- Set temporary minimum of depth and time ---- */
  minX = maxX = data[0].x;
  minY = maxY = data[0].y;

  /* ---- Search for maxima and minima in data ---- */
  for (unsigned long i = 1; i < data.size(); i++) {

   /* ---- Check minX ---- */
   if (data[i].x < minX) minX = data[i].x;

   /* ---- Check maxX ---- */
   if (data[i].x > maxX) maxX = data[i].x;

   /* ---- Check minY ---- */
   if (data[i].y < minY) minY = data[i].y;

   /* ---- Check maxY ---- */
   if (data[i].y > maxY) maxY = data[i].y;

  }

 } else {

  /* ---- Use standard interval ---- */
  minX = minY = (double) 0.0;
  maxX = maxY = (double) 1.0;

 }

#ifndef NDEBUG
 cerr << "X - Range : " << minX << " - " << maxX << endl;
 cerr << "Y - Range : " << minY << " - " << maxY << endl;
#endif

 /* ---- Calc range to be shown and numbers of ticks ---- */
 calcAxisInfo(minX, maxX, xticks, xbigTicks);
 calcAxisInfo(minY, maxY, yticks, ybigTicks);
 
 /* ---- Return a dummy handle ---- */
 return 0;

}

void CS::DrawData(bool doRefresh)
{
 GdkRectangle rec;
 double x1,x2,y1,y2;
 unsigned long i;

 /* ---- Calculate the consts need for conversion to screen coordinates ---- */
 mX = (ux - static_cast<double>(xOrigin)) / (maxX - minX);
 mY = (uy - static_cast<double>(yOrigin)) / (maxY - minY);

 /* ---- Delete anything inside the rectangle ---- */
 pix.draw_rectangle(blackGC, 
                    TRUE, 
                    xOrigin + 1, 
                    uy + 1, 
                    axisWidth - 2,
                    axisHeight - 2); 

 /* ---- Check for more than on entry in curve ---- */
 if (toPlot.size() > 1) {
 
  /* ---- Copy data of first element ---- */
  x1 = toPlot[0].x;
  y1 = toPlot[0].y;  

  /* ---- Iterate for beginning to end of vector ---- */
  for (i = 1; i != toPlot.size(); ++i) {

   /* ---- Goto next item and copy data ---- */
   x2 = toPlot[i].x;
   y2 = toPlot[i].y;

   /* ---- Check for valid range ---- */
   if ((x1 >= minX) && (x1 <= maxX) && (y1 >= minY) && (y1 <= maxY) &&
       (x2 >= minX) && (x2 <= maxX) && (y2 >= minY) && (y2 <= maxY)) {

#ifndef NDEBUG
    cerr << "Draw unmirrored" << endl;
#endif

    /* ---- Draw a line between (x1|y1) and (x2|y2) ---- */
    pix.draw_line(whiteGC, 
                  ConvertX(x1),
                  ConvertY(y1),
                  ConvertX(x2),
                  ConvertY(y2));

   }

   /* ---- Move data (Avoid STL code) ---- */ 
   x1 = x2; y1 = y2;

  }
 
 } else if (toPlot.size() > 0) {

  cerr << "Only one point" << endl;

 }

 /* ---- Should we do a refresh ---- */
 if (doRefresh) {

  /* ---- Complete refresh of widget ---- */
  rec.x = 0; rec.y = 0;
  rec.width = width(); rec.height = height();

  /* ---- Call expose event ---- */
  draw(&rec);

 }

}

void CS::redraw()
{
 /* ---- Fill the pixmap using color black ---- */
 pix.draw_rectangle(blackGC, TRUE, 0, 0, width(), height()); 

 /* ---- Draw the border ---- */
 DrawBorder(false);

 /* ---- Draw the data ---- */
 DrawData(false);

 /* ---- Draw x-axis and y-axis decoration ---- */
 DrawAxisX(false);
 DrawAxisY(false);

 /* ---- Draw the labels ---- */
 DrawLabelsX(false);
 DrawLabelsY(false);

 /* ---- Refresh the widget ---- */
 refresh();

}

void CS::CalcFont()
{
 long distance; 
 long pt;
 const long fontDesLen = 128;
 char fontDes[fontDesLen];

 /* ---- Calculate the smallest distance between two ticks ---- */
 if (width() < height()) {

  /* ---- X - axis is the short one ---- */
  distance = (long) ((double) 0.7 * (double) axisWidth / (double) xticks); 

  /* ---- Check for space to right border ---- */
  long scaledRightBorder = static_cast<long>(0.7 * (width() - ux));
  if (distance >= scaledRightBorder) 
   distance = scaledRightBorder;

 } else {

  /* ---- Y - axis is the short one ---- */
  distance = (long) ((double) 0.7 * (double) axisHeight / (double) yticks);

  /* ---- Check for space to upper border ---- */
  long scaledUpperBorder = static_cast<long>(0.7 * uy);
  if (distance >= scaledUpperBorder) 
   distance = scaledUpperBorder;

 }

 /* ---- Get name of plot font by asking the application object ---- */
 string fontName = app -> getPlotFontName();

 /* ---- Calculate font size ---- */
 pt = 20;
 while (pt > 8) {

  /* ---- Close the font ---- */
  if (font.connected()) font.release();
  
  /* ---- Calc font with height 'pt' ---- */
  snprintf(fontDes, 
           fontDesLen, 
           fontName.c_str(),
           pt);

#ifndef NDEBUG
  cerr << "Try to use font >>" << fontDes << "<<" << endl;
#endif

  /* ---- Load the font ---- */
  font.load(fontDes);

  /* ---- Not possible to create the font (try "fixed") ---- */
  if (!font.connected()) {

   /* ---- Give a warning ---- */
   cerr << "Unable to open " << fontDes << "try 'fixed'" << endl;

   /* --- Try to use "fixed" ---- */
   font.load("fixed");

   /* ---- Check if we can open 'fixed' ---- */
   if (!font.connected()) {

    /* ---- Give a message and quit ---- */
    cerr << "Can't open font " << fontDes << " and font 'fixed'" << endl;
    exit(EXIT_FAILURE);

   }

   /* ---- We have to work with 'fixed' ---- */
   break;

  }

  /* ---- Check if font is small enough ---- */
  static char probeStr[] = "88.8";
  if (distance >= font.text_width(probeStr, strlen(probeStr))) break;

  /* ---- Try next font size ---- */
  pt--;

 }

}

void CS::allocAll()
{
#ifndef NDEBUG
 cerr << "allocAll" << endl;
#endif

 /* ---- Get the colormap and window of this widget ---- */
 colorMap = get_colormap();
 win = get_window();
  
}

gint CS::configure_event_impl(GdkEventConfigure *)
{
#ifndef NDEBUG
 cerr << "Configure event" << endl;
#endif
 
 /* ---- Alloc the needed GC's (if necessary) ---- */
 if (allocGC) allocAll();
 allocGC = false;

 /* ---- Check if a pixmap exists and free if needed ---- */
 /* ---- In Gtk-- version to come call pix.release() ---- */
 if (pix.connected()) pix.release();

 /* ---- Create the pixmap with correct size ---- */
 pix.create(win, width(), height()); 

 /* ---- Is 'whiteGC' currently connected? ---- */
 if (whiteGC.connected()) whiteGC.release();

 /* ---- Create new "white" gc ---- */
 whiteGC = get_style() -> get_fg_gc(GTK_STATE_NORMAL);

 /* ---- Is 'blackGC' currently connected? ---- */
 if (blackGC.connected()) blackGC.release();

 /* ---- Create new "black" gc ---- */
 blackGC = get_style() -> get_bg_gc(GTK_STATE_NORMAL);

 /* ---- Create a gc for pixmap without any modifications ---- */
 if (pixGC.connected()) pixGC.release();
 pixGC = Gdk_GC(pix);

 /* ---- Check the creation ---- */
 if (!static_cast<const Gdk_Pixmap>(pix)) exit(EXIT_FAILURE);

 /* ---- Fill the pixmap using color black ---- */
 pix.draw_rectangle(blackGC, TRUE, 0, 0, width(), height()); 

 /* ---- Draw the border ---- */
 DrawBorder(false);

 /* ---- Draw the data ---- */
 DrawData(false);

 /* ---- Draw x-axis and y-axis decoration ---- */
 DrawAxisX(false);
 DrawAxisY(false);

 /* ---- Calculate best font ---- */
 CalcFont();

 /* ---- Draw the labels ---- */
 DrawLabelsX(false);
 DrawLabelsY(false);

 /* ---- Do a refresh ---- */
 refresh();

 /* ---- Everything is fine ---- */
 return TRUE;

}

gint CS::expose_event_impl(GdkEventExpose *event)
{
#ifndef NDEBUG
 cerr << "Expose event" << endl;
#endif

 /* ---- Redraw the destroyed area ---- */
 win.draw_pixmap(pixGC,
                 pix,
                 event -> area.x, event -> area.y,
                 event -> area.x, event -> area.y,
                 event -> area.width, event -> area.height);

 return FALSE; 

}

void CS::refresh()
{
 GdkRectangle rec;

 /* ---- Complete refresh of widget ---- */
 rec.x = 0; rec.y = 0;
 rec.width = width(); rec.height = height();

 /* ---- Call expose event ---- */
 draw(&rec);

}

void CS::DrawBorder(bool doRefresh)
{
 const long MaxLeftBorder = 60l;
 const long MaxRightBorder = 60l;
 const long MaxTopBorder = 50l;
 const long MaxBottomBorder = 50l;

 GdkRectangle rec;

 /* ---- Calculate origin (left/bottom corner of border) ---- */
 xOrigin = (long) ceil((double) width()  * 0.5 * (1.0 - xRatio));
 if (xOrigin > MaxLeftBorder) xOrigin = MaxLeftBorder;
 yOrigin = (long) ceil((double) height() * 0.5 * (yRatio + 1.0));
 if (yOrigin <= (height() - MaxBottomBorder)) 
     yOrigin = height() - MaxBottomBorder;

 /* ---- Calculate upper right corner ---- */
 ux = (long) floor((double) width()  * 0.5 * (xRatio + 1.0));
 if (ux <= (width() - MaxRightBorder)) ux = width() - MaxRightBorder;
 uy = (long) floor((double) height() * ((1.0 - yRatio) / 2.0));
 if (uy > MaxTopBorder) uy = MaxTopBorder;

#ifndef NDEBUG
 cerr << "Width " << width() << endl;
 cerr << "Heigth " << height() << endl;
 cerr << "XOrigin " << xOrigin << endl;
 cerr << "YOrigin " << yOrigin << endl;
 cerr << "UX " << ux << endl;
 cerr << "UY " << uy << endl;
#endif

 /* ---- Create a rectangle for graphics update ---- */
 rec.x = xOrigin; rec.y = uy;
 rec.width = (ux - xOrigin); rec.height = (yOrigin - uy);

 /* ---- Save width and heigth of rectangle ---- */
 axisWidth = rec.width;
 axisHeight = rec.height;

 /* ---- Draw the border in pixmap ---- */
 pix.draw_rectangle(whiteGC,
                    FALSE,
                    rec.x, rec.y,
                    rec.width, rec.height);      

 /* ---- Should we do a refresh ---- */
 if (doRefresh) {

  /* ---- Complete refresh of widget ---- */
  rec.x = 0; rec.y = 0;
  rec.width = width(); rec.height = height();

  /* ---- Call expose event ---- */
  draw(&rec);

 }

}

void CS::DrawAxisX(bool doRefresh)
{
 GdkRectangle rec;
 int i;

 /* ---- Draw all ticks ---- */
 for (i = 0 ; i < xticks; i++) {

  /* ---- Check if we have to draw a big tick ---- */
  if (i % xbigTicks != 0) {

#ifdef TICKSSYM

   /* ---- Draw the tick into the pixmap ---- */
   pix.draw_line(whiteGC,
                 xOrigin + (long) (i * (axisWidth / xticks)),
                 yOrigin - (long) (xTickSize * axisHeight),
                 xOrigin + (long) (i * (axisWidth / xticks)),
               	 yOrigin + (long) (xTickSize * axisHeight));
   pix.draw_line(whiteGC,
                 xOrigin + (long) (i * (axisWidth / xticks)),
                 yOrigin - axisHeight - (long) (xTickSize * axisHeight),
                 xOrigin + (long) (i * (axisWidth / xticks)),
               	 yOrigin - axisHeight + (long) (xTickSize * axisHeight));

#else

   /* ---- Draw the tick into the pixmap ---- */
   pix.draw_line(whiteGC,
                 xOrigin + (long) (i * (axisWidth / xticks)),
                 yOrigin - (long) (xTickSize * axisHeight),
                 xOrigin + (long) (i * (axisWidth / xticks)),
               	 yOrigin);
   pix.draw_line(whiteGC,
                 xOrigin + (long) (i * (axisWidth / xticks)),
                 yOrigin - axisHeight,
                 xOrigin + (long) (i * (axisWidth / xticks)),
               	 yOrigin - axisHeight + (long) (xTickSize * axisHeight));

#endif

  } else {

#ifdef TICKSSYM

   /* ---- Draw the big tick into the pixmap ---- */
   pix.draw_line(whiteGC,
                 xOrigin + (long) (i * (axisWidth / xticks)),
                 yOrigin - (long) (xBigTickSize * axisHeight),
                 xOrigin + (long) (i * (axisWidth / xticks)),
               	 yOrigin + (long) (xBigTickSize * axisHeight));
   pix.draw_line(whiteGC,
                 xOrigin + (long) (i * (axisWidth / xticks)),
                 yOrigin - axisHeight - (long) (xBigTickSize * axisHeight),
                 xOrigin + (long) (i * (axisWidth / xticks)),
               	 yOrigin - axisHeight + (long) (xBigTickSize * axisHeight));

#else

   /* ---- Draw the big tick into the pixmap ---- */
   pix.draw_line(whiteGC,
                 xOrigin + (long) (i * (axisWidth / xticks)),
                 yOrigin - (long) (xBigTickSize * axisHeight),
                 xOrigin + (long) (i * (axisWidth / xticks)),
               	 yOrigin);
   pix.draw_line(whiteGC,
                 xOrigin + (long) (i * (axisWidth / xticks)),
                 yOrigin - axisHeight,
                 xOrigin + (long) (i * (axisWidth / xticks)),
               	 yOrigin - axisHeight + (long) (xBigTickSize * axisHeight));

#endif

  }

 }

#ifdef TICKSSYM

 /* ---- Draw the last tick (Big one) ---- */
 pix.draw_line(whiteGC,
               xOrigin + axisWidth,
               yOrigin - (long) (xBigTickSize * axisHeight),
               xOrigin + axisWidth,
               yOrigin + (long) (xBigTickSize * axisHeight));
 pix.draw_line(whiteGC,
               xOrigin + axisWidth,
               yOrigin - axisHeight - (long) (xBigTickSize * axisHeight),
               xOrigin + axisWidth,
               yOrigin - axisHeight + (long) (xBigTickSize * axisHeight));

#endif

 /* ---- Should we do a refresh ---- */
 if (doRefresh) {

  /* ---- Complete refresh of widget ---- */
  rec.x = 0; rec.y = 0;
  rec.width = width(); rec.height = height();

  /* ---- Call expose event ---- */
  draw(&rec);

 }

}

void CS::DrawLabelsX(bool doRefresh)
{
 int i;
 GdkRectangle rec;
 char label[128];

 /* ---- Draw all ticks ---- */
 for (i = 0 ; i < xticks; i++) {

  /* ---- Render the label ---- */
  snprintf(label, 5, "%f", minX + i * (maxX - minX) / xticks);  

  /* ---- Draw the text into the pixmap ---- */
  pix.draw_text(font,
                whiteGC,
                xOrigin + (long) (i * (axisWidth / xticks)) + 3,
                yOrigin + (long) (xTickSize * axisHeight) + 10,
                label,
                strlen(label));
 
 }

 /* ---- Render the last label ---- */
 snprintf(label, 5, "%f", maxX);
 
 /* ---- Draw the text into the pixmap ---- */
 pix.draw_text(font,
               whiteGC,
               xOrigin + axisWidth + 3,
               yOrigin + (long) (xTickSize * axisHeight) + 10,
               label,
               strlen(label));

 /* ---- Should we do a refresh ---- */
 if (doRefresh) {

  /* ---- Complete refresh of widget ---- */
  rec.x = 0; rec.y = 0;
  rec.width = width(); rec.height = height();

  /* ---- Call expose event ---- */
  draw(&rec);

 }

}

void CS::DrawLabelsY(bool doRefresh)
{
 int i;
 GdkRectangle rec;
 char label[128];

 /* ---- Draw all ticks ---- */
 for (i = 0 ; i < yticks; i++) {

  /* ---- Render the label ---- */
  snprintf(label, 5, "%f", minY + i * (maxY - minY) / yticks);  

#ifndef NDEBUG
  cerr << "Write Y Label unmirrored" << endl;
#endif

  /* ---- Draw the text into the pixmap ---- */
  draw_text_vert(font,
                 whiteGC,
                 xOrigin - (long) (yTickSize * axisWidth) - 3,
                 yOrigin - (long) (i * (axisHeight  / yticks)) - 3,
                 label,
                 strlen(label));
 
 }

 /* ---- Render the last label ---- */
 snprintf(label, 5, "%f", maxY);

 /* ---- Draw the text into the pixmap ---- */
 draw_text_vert(font,
                whiteGC,
                xOrigin - (long) (yTickSize * axisWidth) - 3,
                yOrigin - (long) axisHeight - 3,
                label,
                strlen(label));

 /* ---- Should we do a refresh ---- */
 if (doRefresh) {

  /* ---- Complete refresh of widget ---- */
  rec.x = 0; rec.y = 0;
  rec.width = width(); rec.height = height();

  /* ---- Call expose event ---- */
  draw(&rec);

 }

}

void CS::DrawAxisY(bool doRefresh)
{
 GdkRectangle rec;
 int i;

 /* ---- Draw all ticks ---- */
 for (i = 0 ; i < yticks; i++) {

  /* ---- Check if we have to draw a big tick ---- */
  if (i % ybigTicks != 0) {

#ifdef TICKSSYM

   /* ---- Draw the tick into the pixmap ---- */
   pix.draw_line(whiteGC,
                 xOrigin - (long) (yTickSize * axisWidth),
                 yOrigin - (long) (i * (axisHeight / yticks)),
               	 xOrigin + (long) (yTickSize * axisWidth),
                 yOrigin - (long) (i * (axisHeight / yticks)));
   pix.draw_line(whiteGC,
                 xOrigin + axisWidth - (long) (yTickSize * axisWidth),
                 yOrigin - (long) (i * (axisHeight / yticks)),
               	 xOrigin + axisWidth + (long) (yTickSize * axisWidth),
                 yOrigin - (long) (i * (axisHeight / yticks)));

#else

   /* ---- Draw the tick into the pixmap ---- */
   pix.draw_line(whiteGC,
                 xOrigin,
                 yOrigin - (long) (i * (axisHeight / yticks)),
               	 xOrigin + (long) (yTickSize * axisWidth),
                 yOrigin - (long) (i * (axisHeight / yticks)));
   pix.draw_line(whiteGC,
                 xOrigin + axisWidth - (long) (yTickSize * axisWidth),
                 yOrigin - (long) (i * (axisHeight / yticks)),
               	 xOrigin + axisWidth,
                 yOrigin - (long) (i * (axisHeight / yticks)));

#endif

  } else {

#ifdef TICKSSYM

   /* ---- Draw the big tick into the pixmap ---- */
   pix.draw_line(whiteGC,
                 xOrigin - (long) (yBigTickSize * axisWidth),
                 yOrigin - (long) (i * (axisHeight / yticks)),
               	 xOrigin + (long) (yBigTickSize * axisWidth),
                 yOrigin - (long) (i * (axisHeight / yticks)));
   pix.draw_line(whiteGC,
                 xOrigin + axisWidth - (long) (yBigTickSize * axisWidth),
                 yOrigin - (long) (i * (axisHeight / yticks)),
               	 xOrigin + axisWidth + (long) (yBigTickSize * axisWidth),
                 yOrigin - (long) (i * (axisHeight / yticks)));

#else

   /* ---- Draw the big tick into the pixmap ---- */
   pix.draw_line(whiteGC,
                 xOrigin,
                 yOrigin - (long) (i * (axisHeight / yticks)),
               	 xOrigin + (long) (yBigTickSize * axisWidth),
                 yOrigin - (long) (i * (axisHeight / yticks)));
   pix.draw_line(whiteGC,
                 xOrigin + axisWidth - (long) (yBigTickSize * axisWidth),
                 yOrigin - (long) (i * (axisHeight / yticks)),
               	 xOrigin + axisWidth,
                 yOrigin - (long) (i * (axisHeight / yticks)));

#endif

  }

 }

#ifdef TICKSSYM

 /* ---- Draw the last tick (Big one) ---- */
 pix.draw_line(whiteGC,
               xOrigin - (long) (yBigTickSize * axisWidth),
               yOrigin - axisHeight,
               xOrigin + (long) (yBigTickSize * axisWidth),
               yOrigin - axisHeight);
 pix.draw_line(whiteGC,
               xOrigin + axisWidth - (long) (yBigTickSize * axisWidth),
               yOrigin - axisHeight,
               xOrigin + axisWidth + (long) (yBigTickSize * axisWidth),
               yOrigin - axisHeight);

#endif

 /* ---- Should we do a refresh ---- */
 if (doRefresh) {

  /* ---- Complete refresh of widget ---- */
  rec.x = 0; rec.y = 0;
  rec.width = width(); rec.height = height();

  /* ---- Call expose event ---- */
  draw(&rec);

 }

}

gint CS::ConvertX(double x)
{
 /* ----- Calculate the position on screen and return ---- */
 return static_cast<gint>((x - minX) * mX + xOrigin);

}

gint CS::ConvertY(double y)
{
 /* ----- Calculate the position on screen and return ---- */
 return static_cast<gint>((y - minY) * mY + yOrigin);

}

void CS::draw_text_vert(Gdk_Font &font, 
                        Gdk_GC &gc,
                        gint x, gint y,
                        const gchar* text,
                        gint text_length)
{
#ifndef NDEBUG
 int lbearing, rbearing, w, ascent, descent, h;

 /* ---- Get infos about used font ---- */
 gdk_text_extents(font.gdkobj(), 
                  text, text_length,
		  &lbearing, &rbearing,
		  &w, &ascent, &descent);
 h = ascent + descent;

 cerr << "Label geometry: " << w << "x" << h << endl;
#endif

 /* ---- Draw text rotated text ---- */
 gdk_draw_text_rot(pix.gdkobj(),
                   font.gdkobj(),
                   gc.gdkobj(),
                   x,
                   y, 
                   width(), // Geometry of 'pix' == geometry of CS
                   height(),
                   text,
                   text_length,
                   M_PI / 2.0);

}   

/* ---- Init epsilon ---- */
const double CS::epsilon = static_cast<double>(5e-7);
