//-*-C++-*-
#ifndef _CS_
#define _CS_

#include <gdk--/font.h>
#include <gdk--/pixmap.h>
#include <gdk--/visual.h>
#include <gdk--/window.h>

#include <gtk--/drawingarea.h>

#include <vector>


class CS : public Gtk::DrawingArea
{
 public:

  CS();
  virtual ~CS();

  /* ---- Datatype for data to plot ---- */
  typedef struct {
                  double x,y;
                 } Point;
  typedef vector<Point> Curve;

  Curve toPlot;

  /* ---- Set the data ---- */
  virtual int setData(Curve &data);

  /* ---- Redraw the whole widget (Draw all elements newly) ---- */
  void redraw();

  /* ---- Refresh widget (Copy backup pixmap to widget) ---- */
  void refresh();

  /* ---- Anything absolutely smaller then 'epsilon' is equal 0.0 ---- */
  static const double epsilon;

 protected:

  Gdk_Pixmap pix;
  Gdk_GC gc;
  Gdk_Visual visual;
  Gdk_Window win;
  Gdk_Font font;
  Gdk_GC whiteGC; 
  Gdk_GC blackGC;
  Gdk_GC pixGC;
  Gdk_Colormap colorMap;

 protected:

  double minX, maxX;
  double minY, maxY;
  long xticks, yticks;
  long xbigTicks, ybigTicks;
  long xLabelLen, yLabelLen;

 protected:

  /* ---- The size of the coordinate system relativ to drawing_area ---- */
  const static double xRatio = 0.75;
  const static double yRatio = 0.75;

  /* --- The size of the ticks used relativ to coordinate system ---- */
  const static double xTickSize = 0.025;
  const static double xBigTickSize = 0.04;
  const static double yTickSize = 0.015;
  const static double yBigTickSize = 0.025;

 protected:

  long xOrigin;
  long yOrigin;
  long ux;
  long uy; 
  long axisWidth;
  long axisHeight;
 
 private:

  /* ---- Flag needed for 'allocAll' ---- */
  bool allocGC;
  
  /* ---- This methods allocates the needed GC's for the first time ---- */
  void allocAll();

  /* ---- Method called when object is (re)configured ---- */
  gint configure_event_impl(GdkEventConfigure*);

  /* ---- Method called for repainting the widget ---- */
  gint expose_event_impl(GdkEventExpose *event);

 protected:

  /* ---- Draw a border ---- */
  void DrawBorder(bool doRefresh = true);

  /* ---- Calculate best font ---- */
  void CalcFont();
 
  /* ---- Draw axis decoration ---- */
  virtual void DrawLabelsX(bool doRefresh = true);
  void DrawAxisX(bool doRefresh = true);
  void DrawAxisY(bool doRefresh = true);
  virtual void DrawLabelsY(bool doRefresh = true);

  /* ---- Draw data ---- */
  virtual void DrawData(bool doRefresh);
 
  /* ---- Draw text vertically ---- */
  void draw_text_vert(Gdk_Font &font, 
                      Gdk_GC &gc,
                      gint x, gint y,
                      const char* text,
                      gint text_length);

 protected:

  /* ---- Convert a value between minX and maxX to plot coordinates ---- */
  virtual gint ConvertX(double x);

  /* ---- Convert a value between minY and maxY to plot coordinates ---- */
  virtual gint ConvertY(double y);

  /* ---- Constants needed for conversion ---- */
  double mX, mY;

 protected:

  /* ---- Calculate number of ticks and a nice looking range ---- */
  void calcAxisInfo(double& min, double& max, long& ticks, long& bigTicks);

};
#endif
