#include <gdk--/color.h>

#include <vector>
#include <iostream>

#include <ColorRange.h>

ColorRange::ColorRange(int min, 
                       int max, 
                       unsigned int numColors, 
                       Gdk_Color* minColor, 
                       Gdk_Color* maxColor)
{
 /* ---- Check for correct number of colors ---- */
 if (numColors > 0) {

  /* ---- Save the range parameters ---- */ 
  this -> min = min;
  this -> max = max;
  this -> numColors = numColors;

  /* ---- Get the rgb-values for 'minColor' and 'maxColor' ---- */
  double minRed   = static_cast<double>(minColor -> get_red());
  double minGreen = static_cast<double>(minColor -> get_green());
  double minBlue  = static_cast<double>(minColor -> get_blue());
  double maxRed   = static_cast<double>(maxColor -> get_red());
  double maxGreen = static_cast<double>(maxColor -> get_green());
  double maxBlue  = static_cast<double>(maxColor -> get_blue());
  
  /* ---- Allocate enough slots ---- */
  colorArray.resize(numColors);  

  /* ---- Red, Green, Blue values for newly created color ---- */
  gushort r,g,b;

  /* ---- Calculate all colors and allocate them ---- */
  for (unsigned int i = 0; i < numColors; i++) {
  
   /* ---- Calc in between rgb-values (Waste speed but avoid rounding errors) ---- */
   r = static_cast<gushort>((maxRed - minRed) / (numColors - 1) * i + minRed);
   g = static_cast<gushort>((maxGreen - minGreen) / (numColors - 1) * i + minGreen);
   b = static_cast<gushort>((maxBlue - minBlue) / (numColors - 1) * i + minBlue);
 
   /* ---- Create new color ---- */
   colorArray[i] = new Gdk_Color();

   /* ---- Check for success ---- */
   if (colorArray[i] == static_cast<Gdk_Color*>(NULL)) {

    /* ---- Show an error and quit ---- */
    cerr << "Can't allocate enough color cells!" << endl;

    /* ---- Exit with fail-level ---- */
    exit(EXIT_FAILURE);

   }

   /* ---- Now set the correct color-values ---- */
   colorArray[i] -> set_red(r);
   colorArray[i] -> set_green(g);
   colorArray[i] -> set_blue(b);

  }

 }

}

ColorRange::~ColorRange()
{
 /* ----- Deallocate all color-objects ---- */
 for (unsigned int i = 0; i < colorArray.size(); ++i) 
  if (colorArray[i] != static_cast<Gdk_Color*>(NULL)) delete colorArray[i];

}

Gdk_Color* ColorRange::getColor(int value)
{
 /* ---- Return the color ---- */
 return colorArray[calcIndex(value)];

}

Gdk_Color* ColorRange::getColorByIndex(int index)
{
 /* ---- Return the color ---- */
 return colorArray[index];

}

int ColorRange::calcIndex(int value)
{
 /* ---- Check for a valid range ---- */
 if (value <= min) return 0;
 if (value >= max) return colorArray.size();

 /* ---- Cast min,max and numColors to double ---- */
 double fmin       = static_cast<double>(min);
 double fmax       = static_cast<double>(max);
 double fnumColors = static_cast<double>(numColors);

 /* ---- Calculate the corresponding index ---- */
 int index = static_cast<int>((((value - fmin) * (fnumColors - 1.0)) / 
                               (fmax - fmin)) + 0.5);

#ifndef NDEBUG
 /* ---- Write the input ---- */
 cerr << "Value " << value << endl;

 /* ---- Check the index ---- */
 if ((static_cast<double>(index) >= 0) && 
     (static_cast<double>(index) <= colorArray.size() - 1)) {

  /* ---- Give a debug-message ---- */
   cerr << "Index (valid) " << index  << " in range " << min << ".." << max << endl; 

 } else {

  /* ---- Give a debug-message ---- */
   cerr << "Index (invalid) " << index  << " in range " << min << ".." << max << endl; 

 }
#endif

 /* ---- Return the calculated index ---- */
 return index;

}
