//-*-C++-*- 
#ifndef _MAINWINDOW_
#define _MAINWINDOW_

#include <gtk--/accelgroup.h>
#include <gtk--/alignment.h>
#include <gtk--/box.h>
#include <gtk--/label.h>
#include <gtk--/main.h>
#include <gtk--/table.h>
#include <gtk--/window.h>
#include <gtk--/fileselection.h>
#include <gtk--/frame.h>
#include <gtk--/style.h>
#include <gtk--/menubar.h>
#include <gtk--/menu.h>
#include <gtk--/entry.h>

#include <SimpleRequest.h>

#include <UCS.h>
#include <UDCF.h>

class FileSelector : public Gtk::FileSelection 
{
 public:

  FileSelector() : Gtk::FileSelection("Select a file") {}

 private:

  gint delete_event_impl(GdkEventAny*) {hide(); return 1;}

};

class MainWindow : public Gtk::Window {

 private:

  Gtk::VBox* vbox;
  Gtk::Table* table;
  UCS* area;

  /* ---- Style for the labels ---- */
  Gtk::Style* style; 

  /* ---- Labels showing dive info ---- */
  Gtk::Label* dateLabel;
  Gtk::Label* tempLabel;
  Gtk::Label* surfLabel;
  Gtk::Label* timeLabel;
  Gtk::Label* startLabel;
  Gtk::Label* stopLabel;
  Gtk::Label* avgDepthLabel;
  Gtk::Label* maxDepthLabel;

  /* ---- Alignments for info labels ---- */
  Gtk::Alignment* dateLabelA;
  Gtk::Alignment* tempLabelA;
  Gtk::Alignment* surfLabelA;
  Gtk::Alignment* timeLabelA;
  Gtk::Alignment* startLabelA;
  Gtk::Alignment* stopLabelA;
  Gtk::Alignment* avgDepthLabelA;
  Gtk::Alignment* maxDepthLabelA;

  /* ---- A frame for dive information ---- */
  Gtk::Frame* diveFrame;

  /* ---- Widget for status-line ---- */
  Gtk::Entry* statusText;

 private:

  /* ---- Pointers to objects needed for a menu ---- */
  Gtk::Menu* fileMenu;
  Gtk::Menu* diveMenu;
  Gtk::Menu* helpMenu;
  Gtk::MenuBar* menuBar;

 private:

  /* ---- File selector object for loading profiles ---- */
  FileSelector* fileSelLoad;

  /* ---- Load / "OK" and "Cancel" of "load"-file selector ---- */
  void fileOpen();
  void fileOpenOk();
  void fileOpenCancel();

  /* ---- File selector object for saving profiles ---- */
  FileSelector* fileSelSave;

  /* ---- Save / "Ok" and "Cancel" of "save"-file selector ----- */
  void fileSave();
  void fileSaveOk();
  void fileSaveCancel();

 private:

  /* ---- Show the about request ---- */ 
  void about();

 private:

  /* ---- Method for terminating program ---- */
  void quitIt();

 private:

  SimpleRequest* oneButReq;
  SimpleRequest* twoButReq;

 private:

  /* ---- UDCF - handle ---- */
  UDCF* profile;
 
  /* ---- Copy of actual dive ---- */  
  UDCFDive* dive;

  /* ---- Flag signalling whether a valid profile is loaded ---- */
  bool haveValidProfile;

  /* ---- Index of group and dive which are actually shown ---- */
  long actGroup; 
  long actDive;

 public:

  /* ---- Constructor ---- */
  MainWindow();

  /* ---- Destructor ---- */
  ~MainWindow();

  /* ---- Display a dive (Given by actGroup/actDive) ---- */
  void displayDive();

  /* ---- Display all info's of a dive in the status window ---- */
  void showInfos(UDCFDive *dive);

  /* ---- Navigation in groups and dives ---- */
  void nextGroup();
  void previousGroup();
  void nextDive();
  void previousDive();

  /* ---- Set status-line ---- */
  void setStatusLine(const char* info); 

 private:

  /* ---- Copy-CTor is not available ---- */
  MainWindow(const MainWindow* win){};

 private:

 /* ---- Write information about 'sample' on status line ---- */
 void ProcessSampleInfo(const UDCFSample* sample);

 /* ---- Create the menu ---- */
 void makeMenu();

 /* ---- Free the menu ---- */
 void freeMenu();

 /* ---- Private method for handling the delete event ---- */
 gint MainWindow::delete_event_impl(GdkEventAny *);

 private:

  /* ---- Handle the key events ---- */
  gint handleKey(GdkEventKey *event);

};
#endif
