#include <Global.h>

#include <PMain.h>
#include <fstream>
#ifndef NDEBUG
#include <iostream>
#endif

#include <stdlib.h>

namespace Gtk {

/* ---- Init all static flags and data ---- */ 
bool PMain::haveValidFont = false;
string PMain::plotFont = "invalid";

PMain::PMain(int &argc, char **&argv, bool have_locale=false)
      :Gtk::Main(argc, argv, have_locale)
{
 /* ---- Check if we have a valid font ---- */
 if (!haveValidFont) {

  /* ---- Load the font (if possible) ---- */
  if (readFontName(plotFont)) {

   /* ---- Now we have a valid font ---- */
   haveValidFont = true;

  } else {

   /* ---- Set a default font ---- */
   haveValidFont = true;
   plotFont = "-*-helvetica-medium-r-*-*-%ld-*-*-*-*-*-*-*";
 
#ifndef NDEBUG
   cerr << "Can't read plot-font use default" << endl;
#endif

  }

 }

} 

const string PMain::getPlotFontName() const
{
 /* ---- Simply return the font ---- */
 return plotFont;
}

bool PMain::readFontName(string& font)
{
 char* home = getenv("HOME");
 string rcName;
 char ch;

 /* ---- Check for valid homedir ---- */
 if (home == static_cast<char *>(NULL)) return false;

 /* ---- Build filename of per user rcfile ---- */
 rcName = home; rcName += "/"; rcName += PRIVATERCDIR; rcName += "/plotfont";

 /* ---- Create a infile object connected to 'rcName' ---- */
 std::ifstream rcFile(rcName.c_str());

 /* ---- Check if 'rcFile' exists ---- */
 if (!rcFile.is_open()) return false;

 /* ---- Init 'plotFont' with emtpy string ---- */
 plotFont = "";
 
 /* ---- Read the text in 'rcFile' to 'plotFont' ---- */
 while((ch = rcFile.get()) && !rcFile.eof() && (isprint(ch)) && (isprint(ch))) 
  plotFont += ch;

 /* ---- Close the stream ---- */
 rcFile.close();

 /* ---- Everything ok ---- */
 return true;

}

} /* ---- End of namespace ---- */
