//-*-C++-*-  
#include <gdk--/pixmap.h>
#include <gdk--/visual.h>
#include <gdk--/window.h>

#include <gtk--/drawingarea.h>
#include <iostream>
#include <math.h>
#include <vector>

#include <UDCF.h>
#include <UCS.h>

#include <stdio.h>

#ifdef WITH_DMALLOC 
#include<dmalloc.h>
#endif

UCS::UCS() : CS()  
{
 /* ---- Set default values ---- */
 minX = minY = (double) 0.0;
 maxX = (double) 60.0;
 maxY = (double) 60.0;
 xticks = 6;
 yticks = 6;
 xbigTicks = 2;
 ybigTicks = 2;
 xLabelLen = yLabelLen = 3;

 /* --- Metric is the default ---- */
 isMetric = true;

 /* ---- Marker is visible by default ---- */
 markerState = true;

 /* ---- Assign a marker index ---- */
 markerIndex = 0;

 /* ---- Init the alloc flag ---- */
 allocated = false;

 /* ---- Init color entries ---- */
 normColor   = static_cast<Gdk_Color*>(NULL);
 dashColor   = static_cast<Gdk_Color*>(NULL);
 decoColor   = static_cast<Gdk_Color*>(NULL);
 ascentColor = static_cast<Gdk_Color*>(NULL);
 surfColor   = static_cast<Gdk_Color*>(NULL);
 breathColor = static_cast<Gdk_Color*>(NULL);
 rbtColor    = static_cast<Gdk_Color*>(NULL);
 errorColor  = static_cast<Gdk_Color*>(NULL);
 markerColor = static_cast<Gdk_Color*>(NULL);

 /* ---- Init the color-range objects ---- */
 metricColors = static_cast<ColorRange*>(NULL);
 imperialColors = static_cast<ColorRange*>(NULL);

}

UCS::~UCS()
{
#ifndef NDEBUG
 cerr << "Destroy UCS" << endl;
#endif

 /* ---- Delete the GC's ---- */
 if (normGC.connected())   normGC.release();
 if (dashGC.connected())   dashGC.release();
 if (decoGC.connected())   decoGC.release();
 if (ascentGC.connected()) ascentGC.release();
 if (surfGC.connected())   surfGC.release();
 if (breathGC.connected()) breathGC.release();
 if (rbtGC.connected())    rbtGC.release();
 if (errorGC.connected())  errorGC.release();
 if (markerGC.connected()) markerGC.release();

 /* ---- Free the color cells if needed ---- */
 if (normColor   != static_cast<Gdk_Color*>(NULL)) delete normColor;
 if (dashColor   != static_cast<Gdk_Color*>(NULL)) delete dashColor;
 if (decoColor   != static_cast<Gdk_Color*>(NULL)) delete decoColor;
 if (ascentColor != static_cast<Gdk_Color*>(NULL)) delete ascentColor;
 if (surfColor   != static_cast<Gdk_Color*>(NULL)) delete surfColor;
 if (breathColor != static_cast<Gdk_Color*>(NULL)) delete breathColor;
 if (rbtColor    != static_cast<Gdk_Color*>(NULL)) delete rbtColor;
 if (errorColor  != static_cast<Gdk_Color*>(NULL)) delete errorColor;
 if (markerColor != static_cast<Gdk_Color*>(NULL)) delete markerColor;

 /* ---- Mark as invalid ---- */
 normColor   = static_cast<Gdk_Color*>(NULL);
 dashColor   = static_cast<Gdk_Color*>(NULL);
 decoColor   = static_cast<Gdk_Color*>(NULL);
 ascentColor = static_cast<Gdk_Color*>(NULL);
 surfColor   = static_cast<Gdk_Color*>(NULL);
 breathColor = static_cast<Gdk_Color*>(NULL);
 rbtColor    = static_cast<Gdk_Color*>(NULL);
 errorColor  = static_cast<Gdk_Color*>(NULL);
 markerColor = static_cast<Gdk_Color*>(NULL);

#ifndef NDEBUG
 cerr << "End of Destroy UCS" << endl;
#endif

}

int UCS::setData(profile &data)
{
 /* ---- Copy plot data ---- */
 toPlot = data;

 /* ---- Set the "border"-values to 1x1 unit square ---- */
 minX = minY = 0.0;
 maxX = maxY = 1.0;

 /* ---- Reset marker index ---- */
 markerIndex = 0;

 /* ---- Check if we have data ---- */
 if (toPlot.size() > 1 ) {

  /* ---- Search for maxima and minima in data ---- */
  for (unsigned long i = 0; i < toPlot.size(); ++i) {

   /* ---- Check minX ---- */
   if ((toPlot[i].type == ST_TD) && 
       (toPlot[i].data.point.x < minX)) minX = data[i].data.point.x;

   /* ---- Check maxX ---- */
   if ((toPlot[i].type == ST_TD) && 
       (toPlot[i].data.point.x > maxX)) maxX = data[i].data.point.x;

   /* ---- Check minY ---- */
   if ((toPlot[i].type == ST_TD) && 
       (toPlot[i].data.point.y < minY)) minY = data[i].data.point.x;

   /* ---- Check maxY ---- */
   if ((toPlot[i].type == ST_TD) && 
       (toPlot[i].data.point.y > maxY)) maxY = data[i].data.point.y;

  }

 }

#ifndef NDEBUG
 cerr << "UCS: X - Range : " << minX << " - " << maxX << endl;
 cerr << "UCS: Y - Range : " << minY << " - " << maxY << endl;
#endif

 /* ---- Calc range to be shown and numbers of ticks ---- */
 calcAxisInfo(minX, maxX, xticks, xbigTicks);
 calcAxisInfo(minY, maxY, yticks, ybigTicks);                                   
 /* ---- Return a dummy handle ---- */
 return 0;

}

bool UCS::GetMarkerState()
{
 /* ---- Simply return the marker state ---- */
 return markerState;
}

void UCS::SetMarkerState(bool state)
{
 /* ---- If marker becomes invisible remove it from drawing ---- */
 if (!state && markerState) DrawMarker();

 /* ---- Set state ---- */
 markerState = state;

 /* ---- Redraw marker if it becomes visible ---- */
 if (markerState) DrawMarker();

 /* ---- Refresh the drawing area ---- */
 refresh();

}

const UDCFSample* UCS::GetMarkerSample()
{
 /* ---- Check for valid data ('toPlot' was set) ---- */
 if (toPlot.size() == 0) return (UDCFSample *) NULL;

 /* ---- Return a pointer to actual sample ---- */
 return &(toPlot[markerIndex]);

}


void UCS::DrawMarker()
{
 /* ---- If profile data is invalid or marker is not drawn then return ---- */
 if ((toPlot.size() == 0) || (!markerState)) return;

 /* ---- Check for a valid depth in actual sample ---- */
 if (toPlot[markerIndex].type == ST_TD) {

  /* ---- Draw the marker ---- */
  pix.draw_line(markerGC,
                xOrigin,
                ConvertY(toPlot[markerIndex].data.point.y),
                xOrigin + axisWidth,
                ConvertY(toPlot[markerIndex].data.point.y));
  pix.draw_line(markerGC,
                ConvertX(toPlot[markerIndex].data.point.x),
                yOrigin,
                ConvertX(toPlot[markerIndex].data.point.x),
                yOrigin - axisHeight);

 } 

}

void UCS::MoveMarkerLeft()
{
 /* ---- Check visibility of marker ---- */
 if (!markerState) return;

 /* ---- Undraw marker ---- */
 DrawMarker();

 /* ---- Check for valid range of markerIndex ---- */
 if (markerIndex > 0) --markerIndex;

 /* ---- Draw new marker ---- */
 DrawMarker();

 /* ---- Refresh the drawing area ---- */
 refresh();

}

void UCS::MoveMarkerRight()
{
 /* ---- Check visibility of marker ---- */
 if (!markerState) return;

 /* ---- Undraw marker ---- */
 DrawMarker();

 /* ---- Check for valid range of markerIndex ---- */
 if ((toPlot.size() > 0) && (markerIndex < (toPlot.size() - 1))) ++markerIndex;

 /* ---- Draw new marker ---- */
 DrawMarker();

 /* ---- Refresh the drawing area ---- */
 refresh();

}


void UCS::allocAll()
{
 static const int numCells = 16;

 /* ---- Check if we had already allocated the color ---- */
 if (allocated) return;
 allocated = true;

 /* ---- Allocate the colors ----- */
 normColor   = new Gdk_Color("sky blue");
 dashColor   = new Gdk_Color("light blue");
 decoColor   = new Gdk_Color("red");
 ascentColor = new Gdk_Color("orange");
 surfColor   = new Gdk_Color("green");
 breathColor = new Gdk_Color("navy blue");
 rbtColor    = new Gdk_Color("red2");
 errorColor  = new Gdk_Color("red3");
 markerColor = new Gdk_Color("white");

 /* ---- Allocate the color for the color-range ---- */
 Gdk_Color* from = new Gdk_Color("light blue");
 Gdk_Color* to = new Gdk_Color("light red");

 /* ---- Check for an error ---- */
 if ((from == static_cast<Gdk_Color*>(NULL)) || (to == static_cast<Gdk_Color*>(NULL))) { 

  /* ---- Show an error and quit ---- */
  cerr << "Can't allocate another 2 color cells!" << endl;

  /* ---- Exit with fail-level ---- */
  exit(EXIT_FAILURE);

 }

 /* ---- Create the color-ranges (16 colors should be enough) ---- */
 metricColors = new ColorRange(0, 32, numCells, from, to);
 imperialColors = new ColorRange(0, 104, numCells, from, to);

 /* ---- Allocate all color in the actual colormap ---- */
 for (int i = 0; i < numCells; ++i) {

  /* ---- Allocate the colors for metric and imperial mode ---- */
  colorMap.alloc(*(metricColors -> getColorByIndex(i)));
  colorMap.alloc(*(imperialColors -> getColorByIndex(i)));

 }

 /* ---- Free the now unneeded color-cells ---- */
 delete from;
 delete to;

 /* ---- Check for an error while creating the colors ---- */
 if ((normColor   == static_cast<Gdk_Color*>(NULL)) ||
     (dashColor   == static_cast<Gdk_Color*>(NULL)) ||
     (decoColor   == static_cast<Gdk_Color*>(NULL)) ||
     (ascentColor == static_cast<Gdk_Color*>(NULL)) ||
     (surfColor   == static_cast<Gdk_Color*>(NULL)) ||
     (breathColor == static_cast<Gdk_Color*>(NULL)) || 
     (rbtColor    == static_cast<Gdk_Color*>(NULL)) || 
     (errorColor  == static_cast<Gdk_Color*>(NULL)) || 
     (markerColor == static_cast<Gdk_Color*>(NULL))) {

  /* ---- Show an error and quit ---- */
  cerr << "Can't allocate color cells!" << endl;

  /* ---- Exit with fail-level ---- */
  exit(EXIT_FAILURE);
 
 }

 /* ---- Allocate the colors in widget's colormap (See CS.h) ---- */
 colorMap.alloc(*normColor);
 colorMap.alloc(*dashColor);
 colorMap.alloc(*decoColor);
 colorMap.alloc(*ascentColor);
 colorMap.alloc(*surfColor);
 colorMap.alloc(*breathColor);
 colorMap.alloc(*rbtColor);
 colorMap.alloc(*errorColor);
 colorMap.alloc(*markerColor);

 /* ---- Create the different GC's ---- */
 normGC.create(pix);
 dashGC.create(pix);
 decoGC.create(pix);
 ascentGC.create(pix);
 surfGC.create(pix);
 breathGC.create(pix);
 rbtGC.create(pix);
 errorGC.create(pix);
 markerGC.create(pix);

 /* ---- Set XOR-Mode for 'markerGC' ---- */
 markerGC.set_function(GDK_XOR);

 /* ---- Set dashed for 'dashGC' ---- */
 dashGC.set_line_style(GDK_LINE_ON_OFF_DASH);
 dashGC.set_dashes(3, 1);
 
 /* ---- Set the color to the corresponding GC's ---- */
 normGC.set_foreground(*normColor); 
 dashGC.set_foreground(*dashColor); 
 decoGC.set_foreground(*decoColor); 
 ascentGC.set_foreground(*ascentColor); 
 surfGC.set_foreground(*surfColor);
 breathGC.set_foreground(*breathColor);
 rbtGC.set_foreground(*rbtColor);
 errorGC.set_foreground(*rbtColor);
 markerGC.set_foreground(*markerColor);

}

void UCS::DrawLabelsY(bool doRefresh)
{
 int i;
 GdkRectangle rec;
 char label[128];

 /* ---- Draw all ticks ---- */
 for (i = 0 ; i < yticks; ++i) {

  /* ---- Render the label ---- */
  snprintf(label, 5, "%f", minY + i * (maxY - minY) / yticks);  

#ifndef NDEBUG
  cerr << "Write Y Label mirrored " << label << endl;
#endif

  /* ---- Draw the text into the pixmap (mirrored) ---- */
  draw_text_vert(font,
                 whiteGC,
                 xOrigin - (long) (yTickSize * axisWidth) - 3,
                 yOrigin + (long) (i * (axisHeight  / yticks)) - 3
                  - axisHeight,
                 label,
                 strlen(label));

 }

 /* ---- Render the last label ---- */
 snprintf(label, 5, "%f", maxY);

 /* ---- Draw the text into the pixmap (mirrored) ---- */
 draw_text_vert(font,
                whiteGC,
                xOrigin - (long) (yTickSize * axisWidth) - 3,
                yOrigin - axisHeight + (long) axisHeight - 3,
                label,
                strlen(label));
 
 /* ---- Should we do a refresh ---- */
 if (doRefresh) {

  /* ---- Complete refresh of widget ---- */
  rec.x = 0; rec.y = 0;
  rec.width = width(); rec.height = height();

  /* ---- Call expose event ---- */
  draw(&rec);

 }

}

void UCS::DrawData(bool doRefresh)
{
 GdkRectangle rec;
 double x1, x2, y1, y2, hx1, hx2, hy2;
 unsigned long i;

 /* ---- Calculate the consts need for conversion to screen coordinates ---- */
 mmX = (ux - static_cast<double>(xOrigin)) / (maxX - minX);
 mmY = (static_cast<double>(yOrigin) - uy) / (maxY - minY);

 /* ---- Alloc the colors if needed ---- */
 allocAll();

 /* ---- Delete anything inside the rectangle ---- */
 pix.draw_rectangle(blackGC, 
                    TRUE, 
                    xOrigin + 1, 
                    uy + 1, 
                    axisWidth - 2,
                    axisHeight - 2);

 /* ---- Check for more than one entry in curve ---- */
 if (toPlot.size() > 1) {

#ifndef NDEBUG
  cerr << "Draw the profile" << endl;
#endif

  /* ---- Init coordinates of data points ---- */
  x1 = x2 = y1 = y2 = 0.0;

  /* ---- Iterate for beginning to end of vector ---- */
  for (i = 0; i < toPlot.size(); ++i) {

   /* ---- Check for ascent warning ---- */
   if ((toPlot[i].type == ST_MARK) && (toPlot[i].data.mark == C_ASC)) {

#ifndef NDEBUG
    cerr << "Found ASCENT warning" << endl;
#endif

    /* ---- Draw a rectangle as special marker ---- */
    pix.draw_rectangle(ascentGC,
                       TRUE,
                       ConvertX(x1) - 2,
                       ConvertY(y1) - 2,
                       4,
                       4);

   }

   /* ---- Check for deco warning ---- */
   if ((toPlot[i].type == ST_MARK) && (toPlot[i].data.mark == C_DECO)) {

#ifndef NDEBUG
    cerr << "Found Deco warning" << endl;
#endif

    /* ---- Draw a 4x4 rectangle as special marker ---- */
    pix.draw_rectangle(decoGC,
                       TRUE,
                       ConvertX(x1) - 2,
                       ConvertY(y1) - 2,
                       4,
                       4);
   }

   /* ---- Check for surface mode ---- */
   if ((toPlot[i].type == ST_MARK) && (toPlot[i].data.mark == C_SURF)) {

#ifndef NDEBUG
    cerr << "Found Surface-Mode marker" << endl;
#endif

    /* ---- Check if there exists a successor depth ---- */
    if (((i + 1) >= toPlot.size()) || (toPlot[i + 1].type != ST_TD)) {

     /* ---- Draw the 5x5 special marker at depth 0.0 (surface)  ---- */
     pix.draw_rectangle(surfGC,
                        TRUE,
                        ConvertX(x1) - 2,
                        (gint) yOrigin - axisHeight - 2 ,
                        5,
                        5);

     /* ---- Draw line from x1/y2 to surface marker ---- */
     pix.draw_line(dashGC, 
                   ConvertX(x1),
                   ConvertY(y1),
                   ConvertX(x1),
                   (gint) yOrigin - axisHeight);

     } else {

      /* ---- X position of marker ---- */
      hx1 = (x1 + toPlot[i+1].data.point.x) / 2.0;

      /* ---- Draw the 5x5 special marker in between the two samples ---- */
      pix.draw_rectangle(surfGC,
                         TRUE,
                         ConvertX(hx1) - 2,
                         (gint) yOrigin - axisHeight - 2 ,
                         5,
                         5);

      /* ---- Draw line from x1/y2 to surface marker ---- */
      pix.draw_line(dashGC, 
                    ConvertX(x1),
                    ConvertY(y1),
                    ConvertX(hx1),
                    (gint) yOrigin - axisHeight);

      /* ---- Get position of destination ---- */
      hx2 = toPlot[i + 1].data.point.x;
      hy2 = toPlot[i + 1].data.point.y;

      /* ---- Check for valid range ---- */
      if ((hx1 >= minX) && (hx1 <= maxX) && 
          (y1  >= minY) && (y1  <= maxY) &&
          (hx2 >= minX) && (hx2 <= maxX) && 
          (hy2 >= minY) && (hy2 <= maxY)) {

       /* ---- Draw line from from surface marker to successor ---- */
       pix.draw_line(dashGC,
                      ConvertX(hx1),
                      (gint) yOrigin - axisHeight,
                      ConvertX(hx2),
                      ConvertY(hy2));
     
      }

#ifndef NDEBUG
     else {
      cerr << "Invalid range found (Surface - marker)!" << endl;
     }
#endif

    }

   }

   /* ---- Check for breath warning ---- */
   if ((toPlot[i].type == ST_MARK) && (toPlot[i].data.mark == C_BREATH)) {

#ifndef NDEBUG
    cerr << "Found out-of breath warning" << endl;
#endif

    /* ---- Draw a 4x4 rectangle as special marker ---- */
    pix.draw_rectangle(breathGC,
                       TRUE,
                       ConvertX(x1) - 2,
                       ConvertY(y1) - 2,
                       4,
                       4);
   }

   /* ---- Check for a temperature - switch ---- */
   if (toPlot[i].type == ST_TEMPERATURE) {

#ifndef NDEBUG
    cerr << "Found a change of the temperature " << endl;
#endif

    /* ---- Check for metric/imperial mode ---- */
    if (isMetric) {

     /* ---- Simply set the new color ---- */
     normGC.set_foreground(
      *(metricColors -> getColor(static_cast<int>(toPlot[i].data.temperature + 0.5))));

    } else {

     /* ---- Simply set the new color ---- */
     normGC.set_foreground(
      *(imperialColors -> getColor(static_cast<int>(toPlot[i].data.temperature + 0.5))));

    }

   }

   /* ---- Check for remaining bottom time warning ---- */
   if ((toPlot[i].type == ST_MARK) && (toPlot[i].data.mark == C_RBT)) {

#ifndef NDEBUG
    cerr << "Found remaining bottom time warning" << endl;
#endif

    /* ---- Draw a 4x4 rectangle as special marker ---- */
    pix.draw_rectangle(rbtGC,
                       TRUE,
                       ConvertX(x1) - 2,
                       ConvertY(y1) - 2,
                       4,
                       4);
   }

   /* ---- Check for general error ---- */
   if ((toPlot[i].type == ST_MARK) && (toPlot[i].data.mark == C_ERROR)) {

#ifndef NDEBUG
    cerr << "Found a general error marker" << endl;
#endif

    /* ---- Draw a 4x4 rectangle as special marker ---- */
    pix.draw_rectangle(errorGC,
                       TRUE,
                       ConvertX(x1) - 2,
                       ConvertY(y1) - 2,
                       4,
                       4);
   }

   /* ---- Check for a point ---- */
   if (toPlot[i].type == ST_TD) {

    /* ---- Set coordinates for the point ---- */
    x2 = toPlot[i].data.point.x;
    y2 = toPlot[i].data.point.y;

    /* ---- Check for valid range ---- */
    if ((x1 >= minX) && (x1 <= maxX) && 
        (y1 >= minY) && (y1 <= maxY) &&
        (x2 >= minX) && (x2 <= maxX) && 
        (y2 >= minY) && (y2 <= maxY)) {

     /* ---- Draw a line between (x1|y1) and (x2|y2) (Y axis mirrored) ---- */
     pix.draw_line(normGC, 
                   ConvertX(x1),
                   ConvertY(y1),
                   ConvertX(x2),
                   ConvertY(y2));

    }

#ifndef NDEBUG
     else {
      cerr << "Invalid range found (Depth sample) !" << endl;
    }
#endif

    /* ---- Set target point as new start point ---- */
    x1 = x2; y1 = y2;

   }

  } 

#ifndef NDEBUG
 } else if (toPlot.size() > 0) {

  cerr << "Only one depth sample!" << endl;

 }
#else 
 }
#endif

 /* ---- Draw the marker ---- */
 DrawMarker();

 /* ---- Should we do a refresh ---- */
 if (doRefresh) {

  /* ---- Complete refresh of widget ---- */
  rec.x = 0; rec.y = 0;
  rec.width = width(); rec.height = height();

  /* ---- Call expose event ---- */
  draw(&rec);

 }

}

void UCS::setMetric(bool flag)
{
 /* ---- Simply copy the flag ---- */
 isMetric = flag;

}

gint UCS::ConvertX(double x)
{
 /* ----- Calculate the position on screen and return ---- */
 return static_cast<gint>(((x - minX) * mmX + xOrigin));

}

gint UCS::ConvertY(double y)
{
 /* ----- Calculate the position on screen and return ---- */
 return static_cast<gint>((y - minY) * mmY + uy);

}
