//-*-C++-*-
#ifndef _UCS_
#define _UCS_

#include <gdk/gdk.h>
#include <gdk--.h>

#include <vector>

#include <CS.h>
#include <UDCF.h>
#include <ColorRange.h>

class UCS : public CS
{
 public:

  UCS();
  virtual ~UCS();

  /* ---- Datatype for profile to plot ---- */
  typedef vector<UDCFSample> profile;

  /* ---- Set the profile data ---- */
  int setData(profile &data);

  /* ---- Get the state of profile marker ---- */
  bool GetMarkerState();

  /* ---- Set state of profile marker ---- */
  void SetMarkerState(bool state);

  /* ---- Get a ptr to the sample referenced by marker (NULL if not available) ---- */
  const UDCFSample* GetMarkerSample();

  /* ---- Move marker left and right ---- */
  void MoveMarkerLeft();
  void MoveMarkerRight();

  /* ---- Set metric/imperial mode for temperature to color mapping ---- */
  void setMetric(bool flag);

 protected:

  /* ---- Copy of data to plot ---- */ 
  profile toPlot;
  double dt;

 private:

  /* ---- Metric/Imperial mode flag ---- */
  bool isMetric;

  /* ---- Internal state of marker ---- */
  bool markerState;

  /* ---- Index of Marker in profile data ---- */
  unsigned long markerIndex;

  /* ---- Colors for different special markers in UDCF-Files ---- */
  Gdk_Color* normColor;
  Gdk_Color* dashColor;
  Gdk_Color* decoColor;
  Gdk_Color* ascentColor;
  Gdk_Color* surfColor;
  Gdk_Color* breathColor;
  Gdk_Color* rbtColor;
  Gdk_Color* errorColor;
  Gdk_Color* markerColor;

  /* ---- Graphics contexts for special markers and normal draw mode ---- */
  Gdk_GC normGC;
  Gdk_GC dashGC;
  Gdk_GC ascentGC;
  Gdk_GC decoGC;
  Gdk_GC surfGC;
  Gdk_GC breathGC;
  Gdk_GC rbtGC;
  Gdk_GC errorGC;
  Gdk_GC markerGC;

  /* ---- System's colormap ---- */
  Gdk_Colormap sysColorMap;

  /* ---- Pointers to objects holding a range of uniquely spreaded colors ---- */
  ColorRange* metricColors;
  ColorRange* imperialColors; 

  /* ---- Show if colors already allocated ---- */
  bool allocated;

 protected:

  /* ---- Draw the marker at actual place ---- */
  void DrawMarker();

  /* ---- Convert to plot coordinates (Y-axis mirrored!) ---- */
  virtual gint ConvertX(double x);
  virtual gint ConvertY(double y);

  /* ---- Constants needed for conversion (Y-axis mirrored!) ---- */
  double mmX, mmY;

 protected:

 /* ---- Draw the profile ---- */
 virtual void DrawData(bool doRefresh);

 /* ---- Draw all labels for Y axis ---- */
 virtual void DrawLabelsY(bool doRefresh = true);

 /* ---- Alloc Colors, GC's .... ----- */
 void allocAll();

 private:

  /* ---- Old CS::setData isn't needed anymore (empty implementation)---- */
  int setData(Curve &data){return 0;};

};
#endif
